@@ -49,7 +49,7 @@ def readAIMECinputs(avalancheDir, pathDict, dirName='com1DFA'):
     profileLayer = list(refDir.glob('*aimec*.shp'))
     try:
         message = ('There should be exactly one path_aimec.shp file containing the avalanche path in %s/Inputs/LINES/' %
-            avalancheDir)
+                   avalancheDir)
         assert len(profileLayer) == 1, message
     except AssertionError:
         raise
@@ -114,6 +114,19 @@ def fetchReferenceSimNo(pathDict, cfgSetup):
         pathDict['referenceFile'] = indexRef
         log.info('Reference Simulation is based on %s = %s - closest value found is: %s' %
                  (cfgSetup['varParList'].split('|')[0], cfgSetup['referenceSimValue'], str(colorValues[indexRef])))
+    elif cfgSetup['referenceSimName'] != '':
+        # if no colorParameter info and refeenceSimValue available but referenceSimName is given - set
+        # simulation with referenceSimName in name as referene simulation
+        simFound = False
+        for fName in pathDict[cfgSetup['resType']]:
+            if cfgSetup['referenceSimName'] in str(fName):
+                pathDict['referenceFile'] = pathDict[cfgSetup['resType']].index(fName)
+                log.info('Reference Simulation is based on provided simName: %s' % cfgSetup['referenceSimName'])
+                simFound = True
+                break
+        if not simFound:
+            pathDict['referenceFile'] = 0
+            log.info('Reference Simulation is based on first simulation in folder')
     else:
         pathDict['referenceFile'] = 0
         log.info('Reference Simulation is based on first simulation in folder')
@@ -121,6 +134,39 @@ def fetchReferenceSimNo(pathDict, cfgSetup):
     return pathDict
 
 
+def computeCellSizeSL(cfgSetup, refResultHeader):
+    """ Get the new (s, l) coordinate cell size
+        read by default the reference result file cell size.
+        If a 'cellSizeSL' is specified in cfgSetup then use this one
+
+        Parameters
+        -----------
+        refResultHeader: dict
+            dictionary wiht the raster header
+        cfgSetup: configParser object
+            configuration for aimec - with field cellSizeSL
+
+        Returns
+        --------
+        cellSizeSL: float
+            cell size to be used for the (s, l) coordinates
+    """
+    if cfgSetup['cellSizeSL'] == '':
+        cellSizeSL = refResultHeader['cellsize']
+        log.info('cellSizeSL is read from the reference header and is : %.2f m' % cellSizeSL)
+    else:
+        try:
+            cellSizeSL = cfgSetup.getfloat('cellSizeSL')
+            log.info('cellSizeSL is read from the configuration file and is : %.2f m' % cellSizeSL)
+        except ValueError:
+            message = ('cellSizeSL is read from the configuration file but should be a number, you provided: %s'
+                       % cfgSetup['cellSizeSL'])
+            log.error(message)
+            raise ValueError(message)
+
+    return cellSizeSL
+
+
 def makeDomainTransfo(pathDict, cfgSetup):
     """ Make domain transformation
 
@@ -134,8 +180,9 @@ def makeDomainTransfo(pathDict, cfgSetup):
         dictionary with path to data to analyze
     cfgSetup : configparser
         configparser with ana3AIMEC settings defined in ana3AIMECCfg.ini
-        regarding domain transformation (domain width w, startOfRunoutAreaAngle or
-        interpolation method, resType and referenceFile to get header info)
+        regarding domain transformation (domain width w, and new cellsize,
+        startOfRunoutAreaAngle or interpolation method, resType and
+        referenceFile to get header info)
 
     Returns
     -------
@@ -160,32 +207,26 @@ def makeDomainTransfo(pathDict, cfgSetup):
     """
     # Read input parameters
     demSource = pathDict['demSource']
+    nRef = pathDict['referenceFile']
+    refResultSource = pathDict[cfgSetup['resType']][nRef]
     ProfileLayer = pathDict['profileLayer']
     splitPointSource = pathDict['splitPointSource']
     DefaultName = pathDict['projectName']
 
-    w = float(cfgSetup['domainWidth'])
-    startOfRunoutAreaAngle = float(cfgSetup['startOfRunoutAreaAngle'])
+    w = cfgSetup.getfloat('domainWidth')
+    startOfRunoutAreaAngle = cfgSetup.getfloat('startOfRunoutAreaAngle')
 
-    # get info on reference result file to be analysed - to get info on xllcenter, yllcenter, cellSize
-    nRef = pathDict['referenceFile']
-    rasterSource = pathDict[cfgSetup['resType']][nRef]
     log.debug('Data-file %s analysed and domain transformation done' % demSource)
 
-    # read data header from result file and dem data
+    # read dem and reference result file
     dem = IOf.readRaster(demSource)
-    rasterResult = IOf.readRaster(rasterSource)
-    header = rasterResult['header']
-    xllc = header['xllcenter']
-    yllc = header['yllcenter']
-    cellSize = header['cellsize']
-    rasterdata = rasterResult['rasterData']
+    refResult = IOf.readRaster(refResultSource)
+    # get the cell size for the (s, l) raster
+    cellSizeSL = computeCellSizeSL(cfgSetup, refResult['header'])
     # Initialize transformation dictionary
     rasterTransfo = {}
     rasterTransfo['domainWidth'] = w
-    rasterTransfo['xllc'] = xllc
-    rasterTransfo['yllc'] = yllc
-    rasterTransfo['cellSize'] = cellSize
+    rasterTransfo['cellSizeSL'] = cellSizeSL
 
     # read avaPath
     avaPath = shpConv.readLine(ProfileLayer, DefaultName, dem)
@@ -209,20 +250,14 @@ def makeDomainTransfo(pathDict, cfgSetup):
     # calculate the real area of the new cells as well as the scoord
     rasterTransfo = getSArea(rasterTransfo)
 
-    log.debug('Size of rasterdata- old: %d x %d - new: %d x %d' % (
-        np.size(rasterdata, 0), np.size(rasterdata, 1),
-        np.size(rasterTransfo['gridx'], 0),
-        np.size(rasterTransfo['gridx'], 1)))
-
     ##########################################################################
-    rasterTransfo['header'] = header
-    # put back scale and origin
-    rasterTransfo['s'] = rasterTransfo['s']*cellSize
-    rasterTransfo['l'] = rasterTransfo['l']*cellSize
-    rasterTransfo['gridx'] = rasterTransfo['gridx']*cellSize + xllc
-    rasterTransfo['gridy'] = rasterTransfo['gridy']*cellSize + yllc
-    rasterTransfo['rasterArea'] = rasterTransfo['rasterArea']*cellSize*cellSize
-    # (x,y) coordinates of the resamples avapth (centerline where l = 0)
+    # put back the scale due to the desired cellsize
+    rasterTransfo['s'] = rasterTransfo['s']*cellSizeSL
+    rasterTransfo['l'] = rasterTransfo['l']*cellSizeSL
+    rasterTransfo['gridx'] = rasterTransfo['gridx']*cellSizeSL
+    rasterTransfo['gridy'] = rasterTransfo['gridy']*cellSizeSL
+    rasterTransfo['rasterArea'] = rasterTransfo['rasterArea']*cellSizeSL*cellSizeSL
+    # (x,y) coordinates of the resampled avapth (centerline where l = 0)
     n = np.shape(rasterTransfo['l'])[0]
     indCenter = int(np.floor(n/2))
     rasterTransfo['x'] = rasterTransfo['gridx'][:, indCenter]
@@ -243,7 +278,7 @@ def makeDomainTransfo(pathDict, cfgSetup):
     rasterTransfo['yBetaPoint'] = rasterTransfo['y'][indStartOfRunout]
     rasterTransfo['startOfRunoutAreaAngle'] = angle[indStartOfRunout]
     log.info('Start of run-out area at the %.2f ° point of coordinates (%.2f, %.2f)' %
-        (rasterTransfo['startOfRunoutAreaAngle'], rasterTransfo['xBetaPoint'], rasterTransfo['yBetaPoint']))
+             (rasterTransfo['startOfRunoutAreaAngle'], rasterTransfo['xBetaPoint'], rasterTransfo['yBetaPoint']))
 
     return rasterTransfo
 
@@ -342,13 +377,13 @@ def makeTransfoMat(rasterTransfo):
                 new coord system in the cross direction
     """
     w = rasterTransfo['domainWidth']
-    cellSize = rasterTransfo['cellSize']
+    cellSize = rasterTransfo['cellSizeSL']
     # number of points describing the avaPath
     n_pnt = np.shape(rasterTransfo['DBXr'])[0]
     # Working with no dimentions
     # (the cellsize scaling will be readded at the end)
     # lcoord is the distance from the polyline (cross section)
-    # maximum step should be smaller then the cellsize
+    # the maximum step size should be smaller then the cellsize
     nTot = np.ceil(w/cellSize)
     # take the next odd integer. This ensures that the lcoord = 0 exists
     nTot = int(nTot+1) if ((nTot % 2) == 0) else int(nTot)
@@ -477,7 +512,7 @@ def getSArea(rasterTransfo):
 def transform(fname, rasterTransfo, interpMethod):
     """ Transfer data from old raster to new raster
 
-    Affect value to the points of the new raster (after domain transormation)
+    Assign value to the points of the new raster (after domain transormation)
 
     Parameters
     ----------
@@ -510,8 +545,7 @@ def transform(fname, rasterTransfo, interpMethod):
     iib = len(Points['x'])
     Points, ioob = geoTrans.projectOnRaster(data, Points, interp=interpMethod)
     newData = Points['z'].reshape(n, m)
-    log.debug('Data-file: %s - %d raster values transferred - %d out of original raster bounds!' %
-             (name, iib-ioob, ioob))
+    log.debug('Data-file: %s - %d raster values transferred - %d out of original raster bounds!' % (name, iib-ioob, ioob))
 
     return newData
 
@@ -783,7 +817,7 @@ def analyzeField(rasterTransfo, transformedRasters, dataType, resultsAreaAnalysi
     return resultsAreaAnalysis
 
 
-def analyzeArea(rasterTransfo, runoutLength, data, cfgSetup, pathDict, cfgFlags):
+def analyzeArea(rasterTransfo, runoutLength, data, cfgSetup, pathDict):
     """Compare results to reference.
 
     Compute True positive, False negative... areas.
@@ -801,8 +835,6 @@ def analyzeArea(rasterTransfo, runoutLength, data, cfgSetup, pathDict, cfgFlags)
         as well as the levels for the contour line plot
     pathDict: dict
         path to data to analyse
-    cfgFlags: configparser
-        configparser with plot and write flags
 
     Returns
     -------
@@ -856,25 +888,20 @@ def analyzeArea(rasterTransfo, runoutLength, data, cfgSetup, pathDict, cfgFlags)
         refMask = copy.deepcopy(data[nRef])
         # prepare mask for area resAnalysis
         refMask = np.where(np.isnan(refMask), 0, refMask)
-        refMask = np.where(refMask < thresholdValue, 0, refMask)
-        refMask = np.where(refMask >= thresholdValue, 1, refMask)
+        refMask = np.where(refMask <= thresholdValue, 0, refMask)
+        refMask = np.where(refMask > thresholdValue, 1, refMask)
         # comparison rasterdata with mask
-        log.debug('{: <15} {: <15} {: <15} {: <15} {: <15}'.format(
-            'Sim number ', 'TP ', 'FN ', 'FP ', 'TN'))
-        newRasterData = copy.deepcopy(rasterdata)
+        log.debug('{: <15} {: <15} {: <15} {: <15} {: <15}'.format('Sim number ', 'TP ', 'FN ', 'FP ', 'TN'))
+        compRasterMask = copy.deepcopy(rasterdata)
         # prepare mask for area resAnalysis
-        newRasterData = np.where(np.isnan(newRasterData), 0, newRasterData)
-        newRasterData = np.where(newRasterData < thresholdValue, 0, newRasterData)
-        newRasterData = np.where(newRasterData >= thresholdValue, 1, newRasterData)
+        compRasterMask = np.where(np.isnan(compRasterMask), 0, compRasterMask)
+        compRasterMask = np.where(compRasterMask <= thresholdValue, 0, compRasterMask)
+        compRasterMask = np.where(compRasterMask > thresholdValue, 1, compRasterMask)
 
-        tpInd = np.where((refMask[nStart:] == 1) &
-                         (newRasterData[nStart:] == 1))
-        fpInd = np.where((refMask[nStart:] == 0) &
-                         (newRasterData[nStart:] == 1))
-        fnInd = np.where((refMask[nStart:] == 1) &
-                         (newRasterData[nStart:] == 0))
-        tnInd = np.where((refMask[nStart:] == 0) &
-                         (newRasterData[nStart:] == 0))
+        tpInd = np.where((refMask[nStart:] == 1) & (compRasterMask[nStart:] == 1))
+        fpInd = np.where((refMask[nStart:] == 0) & (compRasterMask[nStart:] == 1))
+        fnInd = np.where((refMask[nStart:] == 1) & (compRasterMask[nStart:] == 0))
+        tnInd = np.where((refMask[nStart:] == 0) & (compRasterMask[nStart:] == 0))
 
         # subareas
         tp = np.nansum(cellarea[tpInd[0] + nStart, tpInd[1]])
@@ -899,18 +926,15 @@ def analyzeArea(rasterTransfo, runoutLength, data, cfgSetup, pathDict, cfgFlags)
         inputs['compData'] = rasterdata
         # masked data for the dataThreshold given in the ini file
         inputs['refRasterMask'] = refMask
-        inputs['newRasterMask'] = newRasterData
+        inputs['compRasterMask'] = compRasterMask
         inputs['i'] = i
 
         # only plot comparisons of simulations to reference
-        if cfgFlags['savePlot']:
-            if i != nRef:
-                compPlotPath = outAimec.visuComparison(rasterTransfo, inputs, pathDict,
-                                        cfgFlags)
-            elif pathDict['numSim'] == 1:
-                compPlotPath = outAimec.visuComparison(rasterTransfo, inputs, pathDict,
-                                        cfgFlags)
-                log.warning('only one simulation, area comparison not meaningful')
+        if i != nRef:
+            compPlotPath = outAimec.visuComparison(rasterTransfo, inputs, pathDict)
+        elif pathDict['numSim'] == 1:
+            compPlotPath = outAimec.visuComparison(rasterTransfo, inputs, pathDict)
+            log.warning('only one simulation, area comparison not meaningfull')
 
     return TP, FN, FP, TN, compPlotPath
 
