@@ -41,7 +41,7 @@ def AIMEC2Report(pathDict, cfg):
     cfgFlags = cfg['FLAGS']
     interpMethod = cfgSetup['interpMethod']
 
-    log.info('Prepare data for post-ptocessing')
+    log.info('Prepare data for post-processing')
     # Make domain transformation
     log.info("Creating new deskewed raster and preparing new raster assignment function")
     rasterTransfo = aT.makeDomainTransfo(pathDict, cfgSetup)
@@ -50,35 +50,32 @@ def AIMEC2Report(pathDict, cfg):
     # TODO: needs to be moved somewhere else
     # read reference file
     nRef = pathDict['referenceFile']
-    rasterSource = pathDict['ppr'][nRef]
+    rasterSource = pathDict[cfgSetup['resType']][nRef]
 
-    pressureRaster = IOf.readRaster(rasterSource)
+    raster = IOf.readRaster(rasterSource)
     slRaster = aT.transform(rasterSource, rasterTransfo, interpMethod)
     inputData = {}
     inputData['slRaster'] = slRaster
-    inputData['xyRaster'] = pressureRaster['rasterData']
-    outAimec.visuTransfo(rasterTransfo, inputData, cfgSetup, pathDict, cfgFlags)
+    inputData['xyRaster'] = raster['rasterData']
+    inputData['xyHeader'] = raster['header']
+    outAimec.visuTransfo(rasterTransfo, inputData, cfgSetup, pathDict)
     #################################################################
 
     # transform pressure_data, depth_data and speed_data in new raster
     newRasters = {}
     # assign pressure data
     log.debug("Assigning pressure data to deskewed raster")
-    newRasters['newRasterPPR'] = aT.assignData(pathDict['ppr'],
-                                                 rasterTransfo, interpMethod)
+    newRasters['newRasterPPR'] = aT.assignData(pathDict['ppr'],  rasterTransfo, interpMethod)
     # assign depth data
     log.debug("Assigning depth data to deskewed raster")
-    newRasters['newRasterPFD'] = aT.assignData(pathDict['pfd'],
-                                              rasterTransfo, interpMethod)
+    newRasters['newRasterPFD'] = aT.assignData(pathDict['pfd'], rasterTransfo, interpMethod)
     # assign speed data
     log.debug("Assigning speed data to deskewed raster")
-    newRasters['newRasterPFV'] = aT.assignData(pathDict['pfv'],
-                                              rasterTransfo, interpMethod)
+    newRasters['newRasterPFV'] = aT.assignData(pathDict['pfv'], rasterTransfo, interpMethod)
 
     # assign dem data
     log.debug("Assigning dem data to deskewed raster")
-    newRasterDEM = aT.assignData([pathDict['demSource']], rasterTransfo,
-                              interpMethod)
+    newRasterDEM = aT.assignData([pathDict['demSource']], rasterTransfo, interpMethod)
     newRasters['newRasterDEM'] = newRasterDEM[0]
 
     # Analyze data
@@ -90,10 +87,10 @@ def AIMEC2Report(pathDict, cfg):
     # -----------------------------------------------------------
     log.info('Visualisation of AIMEC results')
 
-    plotName = outAimec.visuRunoutComp(rasterTransfo, resAnalysis, newRasters, cfgSetup, pathDict, cfgFlags)
+    plotName = outAimec.visuRunoutComp(rasterTransfo, resAnalysis, cfgSetup, pathDict)
     resAnalysis['slCompPlot'] = {'Aimec comparison of mean and max values along path': plotName}
     if cfgFlags.getboolean('flagMass'):
-        plotName = outAimec.visuMass(resAnalysis, pathDict, cfgFlags)
+        plotName = outAimec.visuMass(resAnalysis, pathDict)
         resAnalysis['massAnalysisPlot'] = {'Aimec mass analysis': plotName}
 
     return rasterTransfo, newRasters, resAnalysis
@@ -131,7 +128,7 @@ def mainAIMEC(pathDict, cfg):
     cfgFlags = cfg['FLAGS']
     interpMethod = cfgSetup['interpMethod']
 
-    log.info('Prepare data for post-ptocessing')
+    log.info('Prepare data for post-processing')
     # Make domain transformation
     log.info("Creating new deskewed raster and preparing new raster assignment function")
     rasterTransfo = aT.makeDomainTransfo(pathDict, cfgSetup)
@@ -148,29 +145,26 @@ def mainAIMEC(pathDict, cfg):
     inputData = {}
     inputData['slRaster'] = slRaster
     inputData['xyRaster'] = raster['rasterData']
-    outAimec.visuTransfo(rasterTransfo, inputData, cfgSetup, pathDict, cfgFlags)
+    inputData['xyHeader'] = raster['header']
+    outAimec.visuTransfo(rasterTransfo, inputData, cfgSetup, pathDict)
     #################################################################
 
     # transform pressure_data, depth_data and speed_data in new raster
     newRasters = {}
     # assign pressure data
     log.debug("Assigning pressure data to deskewed raster")
-    newRasters['newRasterPPR'] = aT.assignData(pathDict['ppr'],
-                                                 rasterTransfo, interpMethod)
+    newRasters['newRasterPPR'] = aT.assignData(pathDict['ppr'], rasterTransfo, interpMethod)
     # assign depth data
     log.debug("Assigning depth data to deskewed raster")
-    newRasters['newRasterPFD'] = aT.assignData(pathDict['pfd'],
-                                              rasterTransfo, interpMethod)
+    newRasters['newRasterPFD'] = aT.assignData(pathDict['pfd'], rasterTransfo, interpMethod)
     # assign speed data
     if pathDict['pfv']:
         log.debug("Assigning speed data to deskewed raster")
-        newRasters['newRasterPFV'] = aT.assignData(pathDict['pfv'],
-                                                  rasterTransfo, interpMethod)
+        newRasters['newRasterPFV'] = aT.assignData(pathDict['pfv'], rasterTransfo, interpMethod)
 
     # assign dem data
     log.info("Assigning dem data to deskewed raster")
-    newRasterDEM = aT.assignData([pathDict['demSource']], rasterTransfo,
-                              interpMethod)
+    newRasterDEM = aT.assignData([pathDict['demSource']], rasterTransfo, interpMethod)
     newRasters['newRasterDEM'] = newRasterDEM[0]
 
     # Analyze data
@@ -181,13 +175,14 @@ def mainAIMEC(pathDict, cfg):
     # result visualisation + report
     # -----------------------------------------------------------
     log.info('Visualisation of AIMEC results')
-    outAimec.visuSimple(rasterTransfo, resAnalysis, newRasters, pathDict, cfgFlags)
+    outAimec.visuSimple(rasterTransfo, resAnalysis, newRasters, pathDict)
     if pathDict['numSim'] == 2:
-        outAimec.visuRunoutComp(rasterTransfo, resAnalysis, newRasters, cfgSetup, pathDict, cfgFlags)
-        if cfgFlags.getboolean('flagMass'):
-            outAimec.visuMass(resAnalysis, pathDict, cfgFlags)
+        outAimec.visuRunoutComp(rasterTransfo, resAnalysis, cfgSetup, pathDict)
     else:
-        outAimec.visuRunoutStat(rasterTransfo, resAnalysis, newRasters, cfgSetup, pathDict, cfgFlags)
+        outAimec.visuRunoutStat(rasterTransfo, resAnalysis, newRasters, cfgSetup, pathDict)
+
+    if cfgFlags.getboolean('flagMass'):
+        outAimec.visuMass(resAnalysis, pathDict)
     outAimec.resultVisu(cfgSetup, pathDict, cfgFlags, rasterTransfo, resAnalysis)
 
     # -----------------------------------------------------------
@@ -246,31 +241,29 @@ def AIMECIndi(pathDict, cfg):
     inputData = {}
     inputData['slRaster'] = slRaster
     inputData['xyRaster'] = anaRaster['rasterData']
-    outAimec.visuTransfo(rasterTransfo, inputData, cfgSetup, pathDict, cfgFlags)
+    outAimec.visuTransfo(rasterTransfo, inputData, cfgSetup, pathDict)
     #################################################################
 
     # transform resType_data in new raster
     newRasters = {}
     # assign pressure data
     log.debug("Assigning data to deskewed raster")
-    newRasters['newRaster' + resType.upper()] = aT.assignData(pathDict[resType],
-                                                 rasterTransfo, interpMethod)
+    newRasters['newRaster' + resType.upper()] = aT.assignData(pathDict[resType], rasterTransfo, interpMethod)
     # assign dem data
     log.debug("Assigning dem data to deskewed raster")
-    newRasterDEM = aT.assignData([pathDict['demSource']], rasterTransfo,
-                              interpMethod)
+    newRasterDEM = aT.assignData([pathDict['demSource']], rasterTransfo, interpMethod)
     newRasters['newRasterDEM'] = newRasterDEM[0]
 
     # Analyze data
     log.debug('Analyzing data in path coordinate system')
-    resAnalysis = postProcessAIMECIndi(rasterTransfo, newRasters, cfgSetup, pathDict, cfgFlags)
+    resAnalysis = postProcessAIMECIndi(rasterTransfo, newRasters, cfgSetup, pathDict)
 
     # -----------------------------------------------------------
     # result visualisation + report
     # -----------------------------------------------------------
     log.info('Visualisation of AIMEC results')
 
-    outAimec.visuRunoutStat(rasterTransfo, resAnalysis, newRasters, cfgSetup, pathDict, cfgFlags)
+    outAimec.visuRunoutStat(rasterTransfo, resAnalysis, newRasters, cfgSetup, pathDict)
     outAimec.resultVisu(cfgSetup, pathDict, cfgFlags, rasterTransfo, resAnalysis)
 
     return rasterTransfo, newRasters, resAnalysis
@@ -405,10 +398,12 @@ def postProcessAIMEC(rasterTransfo, newRasters, cfgSetup, pathDict, cfgFlags):
     resultsAreaAnalysis = aT.analyzeField(rasterTransfo, dataSpeed, 'pfv', resultsAreaAnalysis)
 
     # compute runout based on resType
-    runout, runoutMean, elevRel, deltaH = aT.computeRunOut(rasterTransfo, thresholdValue, resultsAreaAnalysis, transformedDEMRasters)
+    runout, runoutMean, elevRel, deltaH = aT.computeRunOut(rasterTransfo, thresholdValue, resultsAreaAnalysis,
+                                                           transformedDEMRasters)
 
     runoutLength = runout[0]
-    TP, FN, FP, TN, compPlotPath = aT.analyzeArea(rasterTransfo, runoutLength, resultsAreaAnalysis[resType]['transformedRasters'], cfgSetup, pathDict, cfgFlags)
+    TP, FN, FP, TN, compPlotPath = aT.analyzeArea(
+        rasterTransfo, runoutLength, resultsAreaAnalysis[resType]['transformedRasters'], cfgSetup, pathDict)
 
     # affect values to output dictionary
     resAnalysis = {}
@@ -442,7 +437,7 @@ def postProcessAIMEC(rasterTransfo, newRasters, cfgSetup, pathDict, cfgFlags):
     return resAnalysis
 
 
-def postProcessAIMECIndi(rasterTransfo, newRasters, cfgSetup, pathDict, cfgFlags):
+def postProcessAIMECIndi(rasterTransfo, newRasters, cfgSetup, pathDict):
     """ Analyse one peak field transformed data
 
     Analyse peak field set in resType e.g. ppr
@@ -460,8 +455,6 @@ def postProcessAIMECIndi(rasterTransfo, newRasters, cfgSetup, pathDict, cfgFlags
         parameters for data analysis
     pathDict: dict
         path to data to analyse
-    cfgFlags: congfigParser object
-        configuration settings flagMass used
 
     Returns
     -------
@@ -481,10 +474,11 @@ def postProcessAIMECIndi(rasterTransfo, newRasters, cfgSetup, pathDict, cfgFlags
     resultsAreaAnalysis = aT.analyzeField(rasterTransfo, dataResType, resType, resultsAreaAnalysis)
 
     # compute runout based on resType
-    runout, runoutMean, elevRel, deltaH = aT.computeRunOut(rasterTransfo, thresholdValue, resultsAreaAnalysis, transformedDEMRasters)
+    runout, runoutMean, elevRel, deltaH = aT.computeRunOut(rasterTransfo, thresholdValue, resultsAreaAnalysis,
+                                                           transformedDEMRasters)
 
     runoutLength = runout[0]
-    TP, FN, FP, TN, compPlotPath = aT.analyzeArea(rasterTransfo, runoutLength, dataResType, cfgSetup, pathDict, cfgFlags)
+    TP, FN, FP, TN, compPlotPath = aT.analyzeArea(rasterTransfo, runoutLength, dataResType, cfgSetup, pathDict)
 
     # affect values to output dictionary
     resAnalysis = {}
@@ -517,13 +511,15 @@ def aimecRes2ReportDict(resAnalysis, reportD, benchD, refSim):
     else:
         log.warning('Reference File out of range - needs to be 0 or 1')
 
-    reportD['Aimec analysis'] ={'type': 'list', 'runout [m]': resAnalysis['runout'][0][compSim],
-    'max peak pressure [kPa]': resAnalysis['MMPPR'][compSim], 'max peak flow depth [m]': resAnalysis['MMPFD'][compSim],
-    'max peak flow velocity [ms-1]': resAnalysis['MMPFV'][compSim]}
+    reportD['Aimec analysis'] = {'type': 'list', 'runout [m]': resAnalysis['runout'][0][compSim],
+                                 'max peak pressure [kPa]': resAnalysis['MMPPR'][compSim],
+                                 'max peak flow depth [m]': resAnalysis['MMPFD'][compSim],
+                                 'max peak flow velocity [ms-1]': resAnalysis['MMPFV'][compSim]}
 
-    benchD['Aimec analysis'] ={'type': 'list', 'runout [m]': resAnalysis['runout'][0][refSim],
-    'max peak pressure [kPa]': resAnalysis['MMPPR'][refSim], 'max peak flow depth [m]': resAnalysis['MMPFD'][refSim],
-    'max peak flow velocity [ms-1]': resAnalysis['MMPFV'][refSim]}
+    benchD['Aimec analysis'] = {'type': 'list', 'runout [m]': resAnalysis['runout'][0][refSim],
+                                'max peak pressure [kPa]': resAnalysis['MMPPR'][refSim],
+                                'max peak flow depth [m]': resAnalysis['MMPFD'][refSim],
+                                'max peak flow velocity [ms-1]': resAnalysis['MMPFV'][refSim]}
     # add mass info
     if "relMass" in resAnalysis:
         reportD['Aimec analysis'].update({'Initial mass [kg]': resAnalysis['relMass'][compSim]})
