@@ -18,12 +18,19 @@ from itertools import product
 import avaframe.in2Trans.shpConversion as shpConv
 import avaframe.in3Utils.geoTrans as geoTrans
 import avaframe.com1DFA.timeDiscretizations as tD
+import avaframe.out3Plot.outCom1DFA as outCom1DFA
 import avaframe.com1DFA.DFAtools as DFAtls
 import avaframe.com1DFA.DFAfunctionsCython as DFAfunC
 import avaframe.in2Trans.ascUtils as IOf
 import avaframe.in3Utils.fileHandlerUtils as fU
 from avaframe.in3Utils import cfgUtils
 import avaframe.out3Plot.outDebugPlots as debPlot
+import avaframe.in3Utils.initialiseDirs as inDirs
+import avaframe.com1DFA.deriveParameterSet as dP
+import avaframe.com1DFA.com1DFA as com1DFA
+from avaframe.in1Data import getInput as gI
+from avaframe.out1Peak import outPlotAllPeak as oP
+from avaframe.log2Report import generateReport as gR
 
 #######################################
 # Set flags here
@@ -36,7 +43,110 @@ debugPlot = cfgAVA['FLAGS'].getboolean('debugPlot')
 featLF = False
 
 
-def com1DFAMain(cfg, avaDir, cuSimName, inputSimFiles, outDir, relThField):
+def com1DFAMain(avalancheDir, cfgMain, cfgFile='', relThField='', variationDict=''):
+    """ preprocess information from ini and run all desired simulations, create outputs and reports
+
+        Parameters
+        ------------
+        avalancheDir: str or pathlib Path
+            path to avalanche data
+        cfgFile: str or pathlib Path
+            path to configuration file if overwrite is desired
+        variationDict: dict
+            dictionary with parameter variation info if not provided via ini file
+
+        Returns
+        --------
+        particlesList: list
+            list of particles dictionaries for saving time steps
+        fieldsList: list
+            list of fields dictionaries for saving time steps
+        tSave: list
+            list of saving time steps
+        dem: dict
+            dictionary with dem header and raster data (that has been used for computations)
+        plotDict: dict
+            information on result plot paths
+        reportDictList: list
+            list of report dictionaries for all performed simulations
+    """
+
+    modName = 'com1DFA'
+    # Create output and work directories
+    workDir, outDir = inDirs.initialiseRunDirs(avalancheDir, modName)
+
+    # get information on simulations that shall be performed according to parameter variation
+    modCfg, variationDict = dP.getParameterVariationInfo(avalancheDir, com1DFA, cfgFile, variationDict)
+
+    # check if parameter variation on release or entrainment thickness is working - where thickness is read from
+    dP.checkRelEntThVariation(modCfg, variationDict)
+
+    # fetch input data - dem, release-, entrainment- and resistance areas
+    inputSimFiles = gI.getInputDataCom1DFA(avalancheDir, modCfg['FLAGS'])
+
+    # write full configuration file to file
+    cfgUtils.writeCfgFile(avalancheDir, com1DFA, modCfg, fileName='sourceConfiguration')
+
+    # create a list of simulations and generate an individual configuration object for each simulation
+    # if need to reproduce exactly the hash - need to be strings with exactely the same number of digits!!
+    simDict = com1DFA.prepareVarSimDict(modCfg, inputSimFiles, variationDict)
+
+    log.info('The following simulations will be performed')
+    for key in simDict:
+        log.info('Simulation: %s' % key)
+
+    reportDictList = []
+    # loop over all simulations
+    for cuSim in simDict:
+
+        # load configuration dictionary for cuSim
+        cfg = simDict[cuSim]['cfgSim']
+
+        # save configuration settings for each simulation
+        simHash = simDict[cuSim]['simHash']
+        cfgUtils.writeCfgFile(avalancheDir, com1DFA, cfg, fileName=cuSim)
+
+        # log simulation name
+        log.info('Run simulation: %s' % cuSim)
+
+        # set release area scenario
+        inputSimFiles['releaseScenario'] = simDict[cuSim]['relFile']
+
+        # +++++++++++++++++++++++++++++++++
+        # ------------------------
+        particlesList, fieldsList, tSave, dem, reportDict, cfgFinal = com1DFA.com1DFACore(cfg, avalancheDir, cuSim,
+            inputSimFiles, outDir, relThField)
+
+        # +++++++++EXPORT RESULTS AND PLOTS++++++++++++++++++++++++
+
+        reportDictList.append(reportDict)
+
+        # export for visulation
+        if cfg['VISUALISATION'].getboolean('writePartToCSV'):
+            outDir = pathlib.Path(avalancheDir, 'Outputs', modName)
+            com1DFA.savePartToCsv(cfg['VISUALISATION']['particleProperties'], particlesList, outDir)
+
+        # create hash to check if config didnt change
+        simHashFinal = cfgUtils.cfgHash(cfgFinal)
+        if simHashFinal != simHash:
+            log.warning('simulation configuration has been changed since start')
+            cfgUtils.writeCfgFile(avalancheDir, com1DFA, cfg, fileName='%s_butModified' % simHash)
+
+    # Set directory for report
+    reportDir = pathlib.Path(avalancheDir, 'Outputs', modName, 'reports')
+    # Generate plots for all peakFiles
+    plotDict = oP.plotAllPeakFields(avalancheDir, cfgMain['FLAGS'], modName, demData=dem)
+    # write report
+    gR.writeReport(reportDir, reportDictList, cfgMain['FLAGS'], plotDict)
+
+    # read all simulation configuration files and return dataFrame and write to csv
+    standardCfg = cfgUtils.getDefaultModuleConfig(com1DFA, toPrint=False)
+    simDF = cfgUtils.createConfigurationInfo(avalancheDir, standardCfg=standardCfg, writeCSV=True)
+
+    return particlesList, fieldsList, tSave, dem, plotDict, reportDictList
+
+
+def com1DFACore(cfg, avaDir, cuSimName, inputSimFiles, outDir, relThField):
     """ Run main com1DFA model
 
     This will compute a dense flow avalanche
@@ -70,7 +180,7 @@ def com1DFAMain(cfg, avaDir, cuSimName, inputSimFiles, outDir, relThField):
     demOri, inputSimLines = prepareInputData(inputSimFiles)
 
     # find out which simulations to perform
-    relName, inputSimLines, badName = prepareRelase(cfg, inputSimFiles['releaseScenario'], inputSimLines)
+    relName, inputSimLines, badName = prepareReleaseEntrainment(cfg, inputSimFiles['releaseScenario'], inputSimLines)
 
     log.info('Perform %s simulation' % cuSimName)
 
@@ -89,6 +199,14 @@ def com1DFAMain(cfg, avaDir, cuSimName, inputSimFiles, outDir, relThField):
     tcpuDFA = '%.2f' % (time.time() - startTime)
     log.info(('cpu time DFA = %s s' % (tcpuDFA)))
 
+    cfgTrackPart = cfg['TRACKPARTICLES']
+    # track particles
+    if cfgTrackPart.getboolean('trackParticles'):
+        particlesList, trackedPartProp, track = trackParticles(cfgTrackPart, demOri, particlesList)
+        if track:
+            outDirData = outDir / 'particles'
+            fU.makeADir(outDirData)
+            outCom1DFA.plotTrackParticle(outDirData, particlesList, trackedPartProp, cfg, dem)
     if 'particles' in cfgGen['resType']:
         # export particles dictionaries of saving time steps
         outDirData = outDir / 'particles'
@@ -106,7 +224,7 @@ def com1DFAMain(cfg, avaDir, cuSimName, inputSimFiles, outDir, relThField):
     return particlesList, fieldsList, Tsave, dem, reportDict, cfg
 
 
-def prepareRelase(cfg, rel, inputSimLines):
+def prepareReleaseEntrainment(cfg, rel, inputSimLines):
     """ get Simulation to run for a given release
 
 
@@ -116,6 +234,8 @@ def prepareRelase(cfg, rel, inputSimLines):
         configuration parameters - keys: relTh, secRelArea, secondaryRelTh
     rel : str
         path to release file
+    inputSimLines: dict
+        dictionary with dictionaries with input data infos (releaseLine, entLine, ...)
 
     Returns
     -------
@@ -130,7 +250,7 @@ def prepareRelase(cfg, rel, inputSimLines):
     # load info
     entResInfo = inputSimLines['entResInfo']
 
-    # Set release areas and simulation name
+    # Set release areas and release thickness
     relName = rel.stem
     simName = relName
     badName = False
@@ -138,12 +258,9 @@ def prepareRelase(cfg, rel, inputSimLines):
         badName = True
         log.warning('Release area scenario file name includes an underscore \
         the suffix _AF will be added for the simulation name')
-    releaseLine = inputSimLines['releaseLine']
-    for k in range(len(releaseLine['d0'])):
-        if releaseLine['d0'][k] == 'None':
-            releaseLine['d0'][k] = cfg['GENERAL'].getfloat('relTh')
-        else:
-            releaseLine['d0'][k] = float(releaseLine['d0'][k])
+
+    # set release thickness
+    releaseLine = setThickness(cfg, inputSimLines['releaseLine'], 'useRelThFromIni', 'relTh')
     inputSimLines['releaseLine'] = releaseLine
     log.debug('Release area scenario: %s - perform simulations' % (relName))
 
@@ -152,21 +269,53 @@ def prepareRelase(cfg, rel, inputSimLines):
             message = 'No secondary release file found'
             log.error(message)
             raise FileNotFoundError(message)
-        secondaryReleaseLine = inputSimLines['secondaryReleaseLine']
-        for k in range(len(secondaryReleaseLine['d0'])):
-            if secondaryReleaseLine['d0'][k] == 'None':
-                secondaryReleaseLine['d0'][k] = cfg['GENERAL'].getfloat('secondaryRelTh')
-            else:
-                secondaryReleaseLine['d0'][k] = float(secondaryReleaseLine['d0'][k])
+        secondaryReleaseLine = setThickness(cfg, inputSimLines['secondaryReleaseLine'], 'useRelThFromIni', 'secondaryRelTh')
     else:
         inputSimLines['entResInfo']['flagSecondaryRelease'] = 'No'
         secondaryReleaseLine = None
-
     inputSimLines['secondaryReleaseLine'] = secondaryReleaseLine
 
+    if entResInfo['flagEnt'] == 'Yes':
+        # set entrainment thickness
+        entLine = setThickness(cfg, inputSimLines['entLine'], 'useEntThFromIni', 'entTh')
+        inputSimLines['entLine'] = entLine
+
     return relName, inputSimLines, badName
 
 
+def setThickness(cfg, lineTh, useThFromIni, typeTh):
+    """ set thickness in line dictionary for release area, entrainment area
+
+    Parameters
+    -----------
+    lineTh: dict
+        dictionary with info on line (e.g. release area line)
+    useThFromIni: bool
+        True if thickness shall be set from ini file
+    typeTh: str
+        type of thickness to be set (e.g. relTh for release thickness -from ini)
+
+    Returns
+    --------
+    lineTh: dict
+        updated dictionary with new key: thickness and thicknessSource
+
+    """
+
+    lineTh['thicknessSource'] = [''] * len(lineTh['thickness'])
+    if cfg.has_option('GENERAL', useThFromIni) is False:
+        log.warning('Using thickness from ini file flag: %s not found, reading thickness from shp file and if not'
+            ' provided setting thickness to %s' % (useThFromIni, cfg['GENERAL'][typeTh]))
+    if cfg['GENERAL'].getboolean(useThFromIni):
+        lineTh['thickness'] = [cfg['GENERAL'].getfloat(typeTh)] * len(lineTh['thickness'])
+        lineTh['thicknessSource'] = ['ini file'] * len(lineTh['thickness'])
+    else:
+        lineTh['thicknessSource'] = ['ini file' if item == 'None' else 'shp file' for item in lineTh['thickness']]
+        lineTh['thickness'] = [cfg['GENERAL'].getfloat(typeTh) if item == 'None' else float(item) for item in lineTh['thickness']]
+
+    return lineTh
+
+
 def prepareInputData(inputSimFiles):
     """ Fetch input data
 
@@ -207,6 +356,7 @@ def prepareInputData(inputSimFiles):
         entResInfo : flag dict
             flag if Yes entrainment and/or resistance areas found and used for simulation
             flag True if a Secondary Release file found and activated
+
     """
 
     # load data
@@ -219,12 +369,14 @@ def prepareInputData(inputSimFiles):
     # get line from release area polygon
     releaseLine = shpConv.readLine(relFile, 'release1', demOri)
     releaseLine['file'] = relFile
+    releaseLine['type'] = 'Release'
 
     # get line from secondary release area polygon
     if entResInfo['flagSecondaryRelease'] == 'Yes':
         secondaryReleaseFile = inputSimFiles['secondaryReleaseFile']
         secondaryReleaseLine = shpConv.readLine(secondaryReleaseFile, '', demOri)
         secondaryReleaseLine['fileName'] = [secondaryReleaseFile]
+        secondaryReleaseLine['type'] = 'Secondary release'
     else:
         secondaryReleaseLine = None
 
@@ -234,6 +386,7 @@ def prepareInputData(inputSimFiles):
         entLine = shpConv.readLine(entFile, '', demOri)
         entrainmentArea = entFile.name
         entLine['fileName'] = entFile
+        entLine['type'] = 'Entrainment'
     else:
         entLine = None
         entrainmentArea = ''
@@ -244,6 +397,7 @@ def prepareInputData(inputSimFiles):
         resLine = shpConv.readLine(resFile, '', demOri)
         resistanceArea = resFile.name
         resLine['fileName'] = resFile
+        resLine['type'] = 'Resistance'
     else:
         resLine = None
         resistanceArea = ''
@@ -291,7 +445,7 @@ def createReportDict(avaDir, logName, relName, inputSimLines, cfgGen, reportArea
     reportST = {}
     reportST = {}
     reportST = {'headerLine': {'type': 'title', 'title': 'com1DFA Simulation'},
-                'avaName': {'type': 'avaName', 'name': avaDir},
+                'avaName': {'type': 'avaName', 'name': str(avaDir)},
                 'simName': {'type': 'simName', 'name': logName},
                 'time': {'type': 'time', 'time': dateTimeInfo},
                 'Simulation Parameters': {
@@ -307,14 +461,14 @@ def createReportDict(avaDir, logName, relName, inputSimLines, cfgGen, reportArea
                 'Density [kgm-3]': cfgGen['rho'],
                 'Friction model': cfgGen['frictModel']},
                 'Release Area': {'type': 'columns', 'Release area scenario': relName, 'Release Area': relDict['Name'],
-                                 'Release thickness [m]': relDict['d0']}}
+                                 'Release thickness [m]': relDict['thickness']}}
 
     if entInfo == 'Yes':
         reportST.update({'Entrainment area':
-                            {'type': 'columns',
-                            'Entrainment area scenario': entrainmentArea,
-                            'Entrainment thickness [m]': cfgGen.getfloat('hEnt'),
-                            'Entrainment density [kgm-3]': cfgGen['rhoEnt']}})
+                         {'type': 'columns',
+                             'Entrainment area scenario': entrainmentArea,
+                             'Entrainment thickness [m]': cfgGen.getfloat('entTh'),
+                             'Entrainment density [kgm-3]': cfgGen['rhoEnt']}})
     if resInfo == 'Yes':
         reportST.update({'Resistance area': {'type': 'columns', 'Resistance area scenario': resistanceArea}})
 
@@ -466,7 +620,8 @@ def initializeSimulation(cfg, demOri, inputSimLines, logName, relThField):
     if len(relThField) == 0:
         # if no release thickness field or function - set release according to shapefile or ini file
         # this is a list of release rasters that we want to combine
-        releaseLine = prepareArea(releaseLine, demOri, np.sqrt(2), thList=releaseLine['d0'], combine=True, checkOverlap=False)
+        releaseLine = prepareArea(releaseLine, demOri, np.sqrt(2), thList=releaseLine['thickness'],
+            combine=True, checkOverlap=False)
     else:
         # if relTh provided - set release thickness with field or function
         releaseLine = prepareArea(releaseLine, demOri, np.sqrt(2), combine=True, checkOverlap=False)
@@ -480,7 +635,7 @@ def initializeSimulation(cfg, demOri, inputSimLines, logName, relThField):
     relAreaActual = np.nansum(relRasterOnes*dem['areaRaster'])
     relAreaProjected = np.sum(csz*csz*relRasterOnes)
     reportAreaInfo = {'Release area info': {'Projected Area [m2]': '%.2f' % (relAreaProjected),
-                      'Actual Area [m2]': '%.2f' % (relAreaActual)}}
+                                            'Actual Area [m2]': '%.2f' % (relAreaActual)}}
 
     # ------------------------
     # initialize simulation
@@ -497,11 +652,13 @@ def initializeSimulation(cfg, demOri, inputSimLines, logName, relThField):
         secondaryReleaseInfo['header'] = demOri['header']
 
         # fetch secondary release areas
-        secondaryReleaseInfo = prepareArea(secondaryReleaseInfo, demOri, np.sqrt(2), thList=secondaryReleaseInfo['d0'], combine=False)
+        secondaryReleaseInfo = prepareArea(secondaryReleaseInfo, demOri, np.sqrt(2),
+                                           thList=secondaryReleaseInfo['thickness'], combine=False)
         # remove overlap with main release areas
         noOverlaprasterList = []
         for secRelRatser, secRelName in zip(secondaryReleaseInfo['rasterData'], secondaryReleaseInfo['Name']):
-            noOverlaprasterList.append(geoTrans.checkOverlap(secRelRatser, relRaster, 'Secondary release ' + secRelName, 'Release', crop=True))
+            noOverlaprasterList.append(geoTrans.checkOverlap(secRelRatser, relRaster, 'Secondary release ' + secRelName,
+                                                             'Release', crop=True))
 
         secondaryReleaseInfo['flagSecondaryRelease'] = 'Yes'
         secondaryReleaseInfo['rasterList'] = noOverlaprasterList
@@ -514,22 +671,24 @@ def initializeSimulation(cfg, demOri, inputSimLines, logName, relThField):
     # initialize entrainment and resistance
     # get info of simType and whether or not to initialize resistance and entrainment
     simTypeActual = cfgGen['simTypeActual']
-    rhoEnt = cfgGen.getfloat('rhoEnt')
-    hEnt = cfgGen.getfloat('hEnt')
-    entrMassRaster, reportAreaInfo = initializeMassEnt(demOri, simTypeActual, inputSimLines['entLine'], reportAreaInfo, thresholdPointInPoly)
+    entrMassRaster, reportAreaInfo = initializeMassEnt(demOri, simTypeActual, inputSimLines['entLine'], reportAreaInfo,
+                                                       thresholdPointInPoly, cfgGen.getfloat('rhoEnt'))
+
     # check if entrainment and release overlap
     entrMassRaster = geoTrans.checkOverlap(entrMassRaster, relRaster, 'Entrainment', 'Release', crop=True)
     # check for overlap with the secondary release area
     if secondaryReleaseInfo['flagSecondaryRelease'] == 'Yes':
         for secRelRaster in secondaryReleaseInfo['rasterList']:
-            entrMassRaster = geoTrans.checkOverlap(entrMassRaster, secRelRaster, 'Entrainment', 'Secondary release ', crop=True)
+            entrMassRaster = geoTrans.checkOverlap(entrMassRaster, secRelRaster, 'Entrainment', 'Secondary release ',
+                crop=True)
     # surfacic entrainment mass available (unit kg/m²)
-    fields['entrMassRaster'] = entrMassRaster*rhoEnt*hEnt
+    fields['entrMassRaster'] = entrMassRaster
     entreainableMass = np.nansum(fields['entrMassRaster']*dem['areaRaster'])
     log.info('Mass available for entrainment: %.2f kg' % (entreainableMass))
 
     log.debug('Initializing resistance area')
-    cResRaster, reportAreaInfo = initializeResistance(cfgGen, demOri, simTypeActual, inputSimLines['resLine'], reportAreaInfo, thresholdPointInPoly)
+    cResRaster, reportAreaInfo = initializeResistance(cfgGen, demOri, simTypeActual, inputSimLines['resLine'],
+                                                      reportAreaInfo, thresholdPointInPoly)
     fields['cResRaster'] = cResRaster
 
     return particles, fields, dem, reportAreaInfo
@@ -599,10 +758,11 @@ def initializeParticles(cfg, releaseLine, dem, logName=''):
             inDirPart = pathlib.Path(avaDir, 'Outputs', 'com1DFAOrig')
 
         searchDir = inDirPart / 'particles'
-        inDirPart = list(searchDir.glob(('*' + cfg['releaseScenario'] + '_' + '*' + cfg['simTypeActual'] + '*')))
+        inDirPart = list(searchDir.glob(
+            ('*' + cfg['releaseScenario'] + '_' + '*' + cfg['simTypeActual'] + '*')))
         if inDirPart == []:
-            messagePart = 'Initialise particles from file - no particles file found for releaseScenario: %s and simType: %s' % \
-                          (cfg['releaseScenario'], cfg['simTypeActual'])
+            messagePart = ('Initialise particles from file - no particles file found for releaseScenario: %s and simType: %s' %
+                          (cfg['releaseScenario'], cfg['simTypeActual']))
             log.error(messagePart)
             raise FileNotFoundError(messagePart)
         elif len(inDirPart) > 1:
@@ -611,8 +771,9 @@ def initializeParticles(cfg, releaseLine, dem, logName=''):
         else:
             inDirPart = inDirPart[0]
 
-        log.info('Initial particle distribution read from file!! %s' % (inDirPart))
-        Particles, TimeStepInfo = readPartFromPickle(inDirPart)
+        log.info('Initial particle distribution read from file!! %s' %
+                 (inDirPart))
+        Particles, timeStepInfo = readPartFromPickle(inDirPart)
         particles = Particles[0]
         Xpart = particles['x']
         Mpart = particles['m']
@@ -637,8 +798,8 @@ def initializeParticles(cfg, releaseLine, dem, logName=''):
             hCell = relRaster[indRely, indRelx]
             volCell = areaRaster[indRely, indRelx] * hCell
             massCell = volCell * rho
-            xpart, ypart, mPart, nPart = placeParticles(massCell, indRelx, indRely, csz,
-                                         massPerPart, rng, cfg['initPartDistType'].lower())
+            xpart, ypart, mPart, nPart = placeParticles(massCell, indRelx, indRely, csz, massPerPart, rng,
+                                                        cfg['initPartDistType'].lower())
             Npart = Npart + nPart
             partPerCell[indRely, indRelx] = nPart
             # initialize particles position, mass, height...
@@ -679,6 +840,18 @@ def initializeParticles(cfg, releaseLine, dem, logName=''):
     if not cfg.getboolean('initialiseParticlesFromFile'):
         particles = checkParticlesInRelease(particles, releaseLine, cfg.getfloat('thresholdPointInPoly'))
 
+    # add a particles ID:
+    # integer ranging from 0 to nPart in the initialisation.
+    # Everytime that a new particle is created, it gets a new ID that is > nID
+    # where nID is the number of already used IDs
+    # (enable tracking of particles even if particles are added or removed)
+    # unique identifier for each particle
+    particles['ID'] = np.arange(particles['Npart'])
+    # keep track of the identifier (usefull to add identifier to newparticles)
+    particles['nID'] = particles['Npart']
+    # keep track of parents (usefull for new particles created after splitting)
+    particles['parentID'] = np.arange(particles['Npart'])
+
     # initialize time
     t = 0
     particles['t'] = t
@@ -818,7 +991,7 @@ def placeParticles(massCell, indx, indy, csz, massPerPart, rng, initPartDistType
     return xpart, ypart, mPart, nPart
 
 
-def initializeMassEnt(dem, simTypeActual, entLine, reportAreaInfo, thresholdPointInPoly):
+def initializeMassEnt(dem, simTypeActual, entLine, reportAreaInfo, thresholdPointInPoly, rhoEnt):
     """ Initialize mass for entrainment
 
     Parameters
@@ -834,6 +1007,8 @@ def initializeMassEnt(dem, simTypeActual, entLine, reportAreaInfo, thresholdPoin
     thresholdPointInPoly: float
         threshold val that decides if a point is in the polygon, on the line or
         very close but outside
+    rhoEnt: float
+        density of entrainment snow
 
     Returns
     -------
@@ -850,13 +1025,15 @@ def initializeMassEnt(dem, simTypeActual, entLine, reportAreaInfo, thresholdPoin
         entrainmentArea = entLine['fileName']
         log.info('Initializing entrainment area: %s' % (entrainmentArea))
         log.info('Entrainment area features: %s' % (entLine['Name']))
-        entLine = prepareArea(entLine, dem, thresholdPointInPoly)
+        entLine = prepareArea(entLine, dem, thresholdPointInPoly, thList=entLine['thickness'])
         entrMassRaster = entLine['rasterData']
         reportAreaInfo['entrainment'] = 'Yes'
     else:
         entrMassRaster = np.zeros((nrows, ncols))
         reportAreaInfo['entrainment'] = 'No'
 
+    entrMassRaster = entrMassRaster * rhoEnt
+
     return entrMassRaster, reportAreaInfo
 
 
@@ -953,6 +1130,9 @@ def DFAIterate(cfg, particles, fields, dem):
     log.debug('using sphOption %s:' % sphOption)
     # desired output fields
     resTypes = fU.splitIniValueToArraySteps(cfgGen['resType'])
+    # add particles to the results type if trackParticles option is activated
+    if cfg.getboolean('TRACKPARTICLES', 'trackParticles'):
+        resTypes = list(set(resTypes + ['particles']))
     # make sure to save all desiered resuts for first and last time step for
     # the report
     resTypesReport = fU.splitIniValueToArraySteps(cfg['REPORT']['plotFields'])
@@ -985,7 +1165,7 @@ def DFAIterate(cfg, particles, fields, dem):
     particles['iterate'] = iterate
     t = particles['t']
     log.debug('Saving results for time step t = %f s', t)
-    fieldsList, particlesList = appendFieldsParticles(fieldsList, particlesList, particles, fields, resTypes)
+    fieldsList, particlesList = appendFieldsParticles(fieldsList, particlesList, particles, fields, resTypesLast)
     # add initial time step to Tsave array
     Tsave = [0]
     # derive time step for first iteration
@@ -1004,11 +1184,9 @@ def DFAIterate(cfg, particles, fields, dem):
         log.debug('Computing time step t = %f s, dt = %f s' % (t, dt))
         # Perform computations
         if featLF:
-            particles, fields, Tcpu, dt = computeLeapFrogTimeStep(
-                cfgGen, particles, fields, dt, dem, Tcpu)
+            particles, fields, Tcpu, dt = computeLeapFrogTimeStep(cfgGen, particles, fields, dt, dem, Tcpu)
         else:
-            particles, fields, Tcpu = computeEulerTimeStep(
-                cfgGen, particles, fields, dt, dem, Tcpu, frictType)
+            particles, fields, Tcpu = computeEulerTimeStep(cfgGen, particles, fields, dt, dem, Tcpu, frictType)
 
         Tcpu['nSave'] = nSave
         particles['t'] = t
@@ -1062,9 +1240,8 @@ def DFAIterate(cfg, particles, fields, dem):
     log.info(('cpu time Neighbour = %s s' % (Tcpu['Neigh'] / nIter)))
     log.info(('cpu time Fields = %s s' % (Tcpu['Field'] / nIter)))
     log.info(('cpu time TimeLoop = %s s' % (Tcpu['TimeLoop'] / nIter)))
-    log.info(('cpu time total other = %s s' %
-              ((Tcpu['Force'] + Tcpu['ForceSPH'] + Tcpu['Pos'] + Tcpu['Neigh'] + Tcpu['Field']) /
-               nIter)))
+    log.info(('cpu time total other = %s s' % ((Tcpu['Force'] + Tcpu['ForceSPH'] + Tcpu['Pos'] + Tcpu['Neigh'] +
+                                               Tcpu['Field']) / nIter)))
     Tsave.append(t-dt)
     fieldsList, particlesList = appendFieldsParticles(fieldsList, particlesList, particles, fields, resTypesLast)
 
@@ -1079,9 +1256,11 @@ def DFAIterate(cfg, particles, fields, dem):
     stopCritPer = cfgGen.getfloat('stopCrit') * 100.
     # update info dict with stopping info for report
     if stopCritNotReached:
-        infoDict.update({'stopInfo': {'Stop criterion': 'end Time reached: %.2f' % avaTime, 'Avalanche run time [s]': '%.2f' % avaTime}})
+        infoDict.update({'stopInfo': {'Stop criterion': 'end Time reached: %.2f' % avaTime,
+                                      'Avalanche run time [s]': '%.2f' % avaTime}})
     else:
-        infoDict.update({'stopInfo': {'Stop criterion': '< %.2f percent of PKE' % stopCritPer, 'Avalanche run time [s]': '%.2f' % avaTime}})
+        infoDict.update({'stopInfo': {'Stop criterion': '< %.2f percent of PKE' % stopCritPer,
+                                      'Avalanche run time [s]': '%.2f' % avaTime}})
 
     return Tsave, particlesList, fieldsList, infoDict
 
@@ -1141,8 +1320,7 @@ def writeMBFile(infoDict, avaDir, logName):
     with open(massDir / ('mass_%s.txt' % logName), 'w') as mFile:
         mFile.write('time, current, entrained\n')
         for m in range(len(t)):
-            mFile.write('%.02f,    %.06f,    %.06f\n' %
-                    (t[m], massTotal[m], massEntrained[m]))
+            mFile.write('%.02f,    %.06f,    %.06f\n' % (t[m], massTotal[m], massEntrained[m]))
 
 
 def computeEulerTimeStep(cfg, particles, fields, dt, dem, Tcpu, frictType):
@@ -1414,7 +1592,8 @@ def prepareArea(line, dem, radius, thList='', combine=True, checkOverlap=True):
         avapath['Name'] = name
         # if relTh is given - set relTh
         if thList != '':
-            log.info('Release feature %s, relTh= %.2f' % (name, thList[i]))
+            log.info('%s feature %s, thickness: %.2f - read from %s' % (line['type'], name, thList[i],
+                     line['thicknessSource'][i]))
             Raster = polygon2Raster(dem['header'], avapath, radius, th=thList[i])
         else:
             Raster = polygon2Raster(dem['header'], avapath, radius)
@@ -1434,7 +1613,7 @@ def prepareArea(line, dem, radius, thList='', combine=True, checkOverlap=True):
                 raise AssertionError(message)
             else:
                 # if there is an overlap, take average of values for the overlapping cells
-                Raster = np.where(((Raster > 0) & (rast > 0)),  (Raster + rast)/2, Raster + rast)
+                Raster = np.where(((Raster > 0) & (rast > 0)), (Raster + rast)/2, Raster + rast)
         else:
             Raster = Raster + rast
     if debugPlot:
@@ -1691,26 +1870,96 @@ def readPartFromPickle(inDir, flagAvaDir=False):
 
     # initialise list of particle dictionaries
     Particles = []
-    TimeStepInfo = []
+    timeStepInfo = []
     for particles in PartDicts:
         particles = pickle.load(open(particles, "rb"))
         Particles.append(particles)
-        TimeStepInfo.append(particles['t'])
+        timeStepInfo.append(particles['t'])
 
-    return Particles, TimeStepInfo
+    return Particles, timeStepInfo
+
+
+def trackParticles(cfgTrackPart, dem, particlesList):
+    """ track particles from initial area
+
+        Find all particles in an initial area. Find the same particles in
+        the other time steps (+ the children if they were splitted).
+        Extract time series of given properties of the tracked particles
+
+        Parameters
+        -----------
+        cfgTrackPart: configParser
+            centerTrackPartPoint : str
+                centerTrackPartPoint of the location of the particles to
+                track (x|y coordinates)
+            radius : str
+                radius of the circle around point
+            particleProperties: str
+                list of particles properties to extract ('x', 'y', 'ux', 'm'...)
+        dem: dict
+            dem dictionary
+        particlesList: list
+            list of particles dictionary
+
+        Returns
+        -------
+        particlesList : list
+            Particles list of dict updated with the 'trackedParticles' array
+            (in the array, ones for particles that are tracked, zeros otherwise)
+        trackedPartProp: dict
+            dictionary with time series of the wanted properties for tracked
+            particles
+        track: boolean
+            False if no particles are tracked
+    """
+
+    # read particle properties to be extracted
+    particleProperties = cfgTrackPart['particleProperties']
+    if particleProperties == '':
+        particleProperties = ['x', 'y', 'z', 'ux', 'uy', 'uz', 'm', 'h']
+    else:
+        particleProperties = set(['x', 'y', 'z', 'ux', 'uy', 'uz', 'm', 'h'] + particleProperties.split('|'))
+    # read location of particle to be tracked
+    radius = cfgTrackPart.getfloat('radius')
+    centerList = cfgTrackPart['centerTrackPartPoint']
+    centerList = centerList.split('|')
+    centerTrackPartPoint = {'x': np.array([float(centerList[0])]),
+                            'y': np.array([float(centerList[1])])}
+    centerTrackPartPoint, _ = geoTrans.projectOnRaster(
+        dem, centerTrackPartPoint, interp='bilinear')
+    centerTrackPartPoint['x'] = (centerTrackPartPoint['x']
+                                 - dem['header']['xllcenter'])
+    centerTrackPartPoint['y'] = (centerTrackPartPoint['y']
+                                 - dem['header']['yllcenter'])
+
+    # start by finding the particles to be tracked
+    particles2Track, track = DFAtls.findParticles2Track(particlesList[0], centerTrackPartPoint, radius)
+    if track:
+        # find those same particles and their children in the particlesList
+        particlesList, nPartTracked = DFAtls.getTrackedParticles(particlesList, particles2Track)
+
+        # extract the wanted properties for the tracked particles
+        trackedPartProp = DFAtls.getTrackedParticlesProperties(particlesList, nPartTracked, particleProperties)
+    else:
+        trackedPartProp = None
+
+    return particlesList, trackedPartProp, track
 
 
 def savePartToCsv(particleProperties, dictList, outDir):
-    """ Save each particle dictionary from a list to a csv file; works also for one dictionary instead of list
+    """ Save each particle dictionary from a list to a csv file;
+        works also for one dictionary instead of list
 
         Parameters
         ---------
         particleProperties: str
-            all particle properties that shall be saved to csv file (e.g.: m, velocityMagnitude, ux,..)
+            all particle properties that shall be saved to csv file
+            (e.g.: m, velocityMagnitude, ux,..)
         dictList: list or dict
             list of dictionaries or single dictionary
         outDir: str
-            path to output directory; particlesCSV will be created in this outDir
+            path to output directory; particlesCSV will be created in this
+            outDir
     """
 
     # set output directory
@@ -1775,14 +2024,14 @@ def exportFields(cfg, Tsave, fieldsList, demOri, outDir, logName):
 
     resTypesGen = fU.splitIniValueToArraySteps(cfg['GENERAL']['resType'])
     resTypesReport = fU.splitIniValueToArraySteps(cfg['REPORT']['plotFields'])
-    if resTypesGen == []:
-        resTypesGen = resTypesReport
     if 'particles' in resTypesGen:
         resTypesGen.remove('particles')
+    if 'particles' in resTypesReport:
+        resTypesReport.remove('particles')
     numberTimes = len(Tsave)-1
     countTime = 0
     for timeStep in Tsave:
-        if (countTime == numberTimes):
+        if (countTime == numberTimes) or (countTime == 0):
             # for last time step we need to add the report fields
             resTypes = list(set(resTypesGen + resTypesReport))
         else:
@@ -1791,22 +2040,26 @@ def exportFields(cfg, Tsave, fieldsList, demOri, outDir, logName):
             resField = fieldsList[countTime][resType]
             if resType == 'ppr':
                 resField = resField * 0.001
-            dataName = logName + '_' + resType + '_' + 't%.2f' % (Tsave[countTime]) + '.asc'
+            dataName = (logName + '_' + resType + '_' + 't%.2f' % (Tsave[countTime]) + '.asc')
             # create directory
             outDirPeak = outDir / 'peakFiles' / 'timeSteps'
             fU.makeADir(outDirPeak)
             outFile = outDirPeak / dataName
-            IOf.writeResultToAsc(demOri['header'], resField, outFile, flip=True)
+            IOf.writeResultToAsc(
+                demOri['header'], resField, outFile, flip=True)
             if countTime == numberTimes:
-                log.debug('Results parameter: %s has been exported to Outputs/peakFiles for time step: %.2f - FINAL time step ' % (resType, Tsave[countTime]))
+                log.debug('Results parameter: %s has been exported to Outputs/peakFiles for time step: %.2f - FINAL time step ' %
+                          (resType, Tsave[countTime]))
                 dataName = logName + '_' + resType + '.asc'
                 # create directory
                 outDirPeakAll = outDir / 'peakFiles'
                 fU.makeADir(outDirPeakAll)
                 outFile = outDirPeakAll / dataName
-                IOf.writeResultToAsc(demOri['header'], resField, outFile, flip=True)
+                IOf.writeResultToAsc(
+                    demOri['header'], resField, outFile, flip=True)
             else:
-                log.debug('Results parameter: %s has been exported to Outputs/peakFiles for time step: %.2f ' % (resType, Tsave[countTime]))
+                log.debug('Results parameter: %s has been exported to Outputs/peakFiles for time step: %.2f ' %
+                          (resType, Tsave[countTime]))
         countTime = countTime + 1
 
 
@@ -1864,7 +2117,8 @@ def prepareVarSimDict(standardCfg, inputSimFiles, variationDict):
             cfgSimObject = cfgUtils.convertDictToConfigParser(cfgSim)
             # create unique hash for simulation configuration
             simHash = cfgUtils.cfgHash(cfgSimObject)
-            simName = relNameSim + '_' + row._asdict()['simTypeList'] + '_' + cfgSim['GENERAL']['modelType'] + '_' + simHash
+            simName = (relNameSim + '_' + row._asdict()['simTypeList'] + '_' + cfgSim['GENERAL']['modelType'] + '_' +
+                       simHash)
             simDict[simName] = {'simHash': simHash, 'releaseScenario': relName,
                                 'simType': row._asdict()['simTypeList'], 'relFile': rel,
                                 'cfgSim': cfgSimObject}
