@@ -44,7 +44,7 @@ def getGeneralConfig():
         raise FileNotFoundError('None of the provided cfg files exist ')
 
     # Finally read it
-    cfg = compareConfig(iniFile, 'General', compare)
+    cfg,_ = compareConfig(iniFile, 'General', compare)
 
     return cfg
 
@@ -89,8 +89,8 @@ def getModuleConfig(module, fileOverride='', modInfo=False):
             iniFile = [defaultFile, fileOverride]
             compare = True
         else:
-            raise FileNotFoundError('Provided fileOverride does not exist: '
-                                    + fileOverride)
+            raise FileNotFoundError('Provided fileOverride does not exist: ' +
+                                    fileOverride)
 
     elif localFile.is_file():
         iniFile = localFile
@@ -103,7 +103,10 @@ def getModuleConfig(module, fileOverride='', modInfo=False):
         raise FileNotFoundError('None of the provided cfg files exist ')
 
     # Finally read it
-    cfg = compareConfig(iniFile, modName, compare, modInfo)
+    cfg, modDict = compareConfig(iniFile, modName, compare, modInfo)
+
+    if modInfo:
+        return cfg, modDict
 
     return cfg
 
@@ -133,7 +136,7 @@ def getDefaultModuleConfig(module, toPrint=True):
     log.debug('defaultFile: %s', defaultFile)
 
     # Finally read it
-    cfg = compareConfig(defaultFile, modName, compare=False, toPrint=toPrint)
+    cfg,_ = compareConfig(defaultFile, modName, compare=False, toPrint=toPrint)
 
     return cfg
 
@@ -205,7 +208,8 @@ def compareConfig(iniFile, modName, compare, modInfo=False, toPrint=True):
                 cfg.add_section(section)
                 log.info('Additional section [\'%s\'] is added to the configuration.' % (section))
                 for key in locCfg.items(section):
-                    log.info('Additional Key [\'%s\'] in section [\'%s\'] is added to the configuration.' % (key[0], section))
+                    log.info('Additional Key [\'%s\'] in section [\'%s\'] is added to the configuration.' %
+                             (key[0], section))
                     cfg.set(section, key[0], key[1])
                     log.info('\t\t%s : %s', key[0], key[1])
 
@@ -219,10 +223,7 @@ def compareConfig(iniFile, modName, compare, modInfo=False, toPrint=True):
         if toPrint:
             logUtils.writeCfg2Log(cfg, modName)
 
-    if modInfo:
-        return cfg, modDict
-    else:
-        return cfg
+    return cfg, modDict
 
 
 def writeCfgFile(avaDir, module, cfg, fileName=''):
@@ -490,6 +491,7 @@ def filterSims(avalancheDir, parametersDict, specDir=''):
 
     return simNameList
 
+
 def orderSimFiles(avalancheDir, inputDir, varParList, ascendingOrder, specDir=''):
     """ Filter simulations results using a list of parameters and a flag if in ascending or descending order
 
@@ -512,7 +514,6 @@ def orderSimFiles(avalancheDir, inputDir, varParList, ascendingOrder, specDir=''
             dataFrame of simulation results (fileName, ... and values for parameters in varParList)
     """
 
-
     # load dataFrame for all configurations
     simDF = createConfigurationInfo(avalancheDir)
     # create dataframe for simulation results in inputDir
